@@ -158,8 +158,6 @@ function iis7_save_url_rewrite_rules(){
 			$rule = $wp_rewrite->iis7_url_rewrite_rules();
 			if ( ! empty($rule) ) {
 				return iis7_add_rewrite_rule($web_config_file, $rule);
-			} else {
-				return iis7_delete_rewrite_rule($web_config_file);
 			}
 		}
 	}
@@ -475,7 +473,7 @@ function iis7_rewrite_rule_exists($filename) {
 	if ( $doc->load($filename) === false )
 		return false;
 	$xpath = new DOMXPath($doc);
-	$rules = $xpath->query('/configuration/system.webServer/rewrite/rules/rule[@name=\'wordpress\']');
+	$rules = $xpath->query('/configuration/system.webServer/rewrite/rules/rule[starts-with(@name,\'wordpress\')]');
 	if ( $rules->length == 0 )
 		return false;
 	else
@@ -504,11 +502,12 @@ function iis7_delete_rewrite_rule($filename) {
 	if ( $doc -> load($filename) === false )
 		return false;
 	$xpath = new DOMXPath($doc);
-	$rules = $xpath->query('/configuration/system.webServer/rewrite/rules/rule[@name=\'wordpress\']');
+	$rules = $xpath->query('/configuration/system.webServer/rewrite/rules/rule[starts-with(@name,\'wordpress\')]');
 	if ( $rules->length > 0 ) {
-		$child = $rules->item(0);
-		$parent = $child->parentNode;
-		$parent->removeChild($child);
+		foreach ($rules as $child) {
+			$parent = $child->parentNode;
+			$parent->removeChild($child);		
+		}
 		$doc->formatOutput = true;
 		saveDomDocument($doc, $filename);
 	}
@@ -543,8 +542,8 @@ function iis7_add_rewrite_rule($filename, $rewrite_rule) {
 
 	$xpath = new DOMXPath($doc);
 
-	// First check if the rule already exists as in that case there is no need to re-add it
-	$wordpress_rules = $xpath->query('/configuration/system.webServer/rewrite/rules/rule[@name=\'wordpress\']');
+	// First check if any of the wordpress rules already exist as in that case there is no need to re-add them
+	$wordpress_rules = $xpath->query('/configuration/system.webServer/rewrite/rules/rule[starts-with(@name,\'wordpress\')]');
 	if ( $wordpress_rules->length > 0 )
 		return true;
 
